@@ -3,14 +3,24 @@ from gym import spaces
 import numpy as np
 import warnings
 
-from .manual_control import manual_control
+from pettingzoo.utils import wrappers
+from pettingzoo.utils.agent_selector import agent_selector
 
 
-class env(AECEnv):
-    metadata = {'render.modes': ['ansi']}
+def env():
+    env = raw_env()
+    env = wrappers.TerminateIllegalWrapper(env, illegal_reward=-1)
+    env = wrappers.AssertOutOfBoundsWrapper(env)
+    env = wrappers.NaNRandomWrapper(env)
+    env = wrappers.OrderEnforcingWrapper(env)
+    return env
+
+
+class raw_env(AECEnv):
+    metadata = {'render.modes': ['human']}
 
     def __init__(self):
-        super(env, self).__init__()
+        super().__init__()
         # 6 rows x 7 columns
         # blank space = 0
         # agent 0 -- 1
@@ -23,14 +33,8 @@ class env(AECEnv):
 
         self.agent_order = list(self.agents)
 
-        self.action_spaces = {i: spaces.Discrete(7) for i in range(2)}
-        self.observation_spaces = {i: spaces.Box(low=0, high=2, shape=(6, 7), dtype=np.int8) for i in range(2)}
-
-        self.rewards = {i: 0 for i in range(self.num_agents)}
-        self.dones = {i: False for i in range(self.num_agents)}
-        self.infos = {i: {'legal_moves': list(range(7))} for i in range(self.num_agents)}
-
-        self.agent_selection = 0
+        self.action_spaces = {i: spaces.Discrete(7) for i in self.agents}
+        self.observation_spaces = {i: spaces.Box(low=0, high=1, shape=(6, 7, 2), dtype=np.int8) for i in self.agents}
 
     # Key
     # ----
@@ -46,44 +50,45 @@ class env(AECEnv):
     #        [2, 0, 0, 0, 1, 1, 0],
     #        [1, 1, 2, 1, 0, 1, 0]], dtype=int8)
     def observe(self, agent):
-        return np.array(self.board).reshape(6, 7)
+        board_vals = np.array(self.board).reshape(6, 7)
+        cur_player = self.agents.index(self.agent_selection)
+        opp_player = (cur_player + 1) % 2
+
+        cur_p_board = np.equal(board_vals, cur_player + 1)
+        opp_p_board = np.equal(board_vals, opp_player + 1)
+        return np.stack([cur_p_board, opp_p_board], axis=2).astype(np.int8)
 
     # action in this case is a value from 0 to 6 indicating position to move on the flat representation of the connect4 board
     def step(self, action, observe=True):
-        # check if input action is a valid move (0 == empty spot)
-        if(self.board[0:7][action] == 0):
-            # valid move
-            for i in list(filter(lambda x: x % 7 == action, list(range(41, -1, -1)))):
-                if self.board[i] == 0:
-                    self.board[i] = self.agent_selection + 1
-                    break
+        # assert valid move
+        assert (self.board[0:7][action] == 0), "played illegal move."
 
-            next_agent = 1 if (self.agent_selection == 0) else 0
+        piece = self.agents.index(self.agent_selection) + 1
+        for i in list(filter(lambda x: x % 7 == action, list(range(41, -1, -1)))):
+            if self.board[i] == 0:
+                self.board[i] = piece
+                break
 
-            # update infos with valid moves
-            self.infos[self.agent_selection]['legal_moves'] = [i for i in range(7) if self.board[i] == 0]
-            self.infos[next_agent]['legal_moves'] = [i for i in range(7) if self.board[i] == 0]
+        next_agent = self._agent_selector.next()
 
-            winner = self.check_for_winner()
+        # update infos with valid moves
+        self.infos[self.agent_selection]['legal_moves'] = [i for i in range(7) if self.board[i] == 0]
+        self.infos[next_agent]['legal_moves'] = [i for i in range(7) if self.board[i] == 0]
 
-            # check if there is a winner
-            if winner:
-                self.rewards[self.agent_selection] += 1
-                self.rewards[next_agent] -= 1
-                self.dones = {i: True for i in range(self.num_agents)}
-            # check if there is a tie
-            elif all(x in [1, 2] for x in self.board):
-                # once either play wins or there is a draw, game over, both players are done
-                self.dones = {i: True for i in range(self.num_agents)}
-            else:
-                # no winner yet
-                self.agent_selection = next_agent
+        winner = self.check_for_winner()
 
+        # check if there is a winner
+        if winner:
+            self.rewards[self.agent_selection] += 1
+            self.rewards[next_agent] -= 1
+            self.dones = {i: True for i in self.agents}
+        # check if there is a tie
+        elif all(x in [1, 2] for x in self.board):
+            # once either play wins or there is a draw, game over, both players are done
+            self.dones = {i: True for i in self.agents}
         else:
-            # invalid move, end game
-            self.rewards[self.agent_selection] -= 1
-            self.dones = {i: True for i in range(self.num_agents)}
-            warnings.warn("Bad connect four move made, game terminating with current player losing. env.infos[player]['legal_moves'] contains a list of all legal moves that can be chosen.")
+            # no winner yet
+            self.agent_selection = next_agent
 
         if observe:
             return self.observe(self.agent_selection)
@@ -94,26 +99,29 @@ class env(AECEnv):
         # reset environment
         self.board = [0] * (6 * 7)
 
-        self.rewards = {i: 0 for i in range(self.num_agents)}
-        self.dones = {i: False for i in range(self.num_agents)}
-        self.infos = {i: {'legal_moves': list(range(7))} for i in range(self.num_agents)}
+        self.rewards = {i: 0 for i in self.agents}
+        self.dones = {i: False for i in self.agents}
+        self.infos = {i: {'legal_moves': list(range(7))} for i in self.agents}
+
+        self._agent_selector = agent_selector(self.agents)
+
+        self.agent_selection = self._agent_selector.reset()
 
-        # selects the first agent
-        self.agent_selection = 0
         if observe:
             return self.observe(self.agent_selection)
         else:
             return
 
     def render(self, mode='ansi'):
-        print(str(self.observe(self.agent_selection)))
+        print("{}'s turn'".format(self.agent_selection))
+        print(str(np.array(self.board).reshape(6, 7)))
 
     def close(self):
         pass
 
     def check_for_winner(self):
         board = np.array(self.board).reshape(6, 7)
-        piece = self.agent_selection + 1
+        piece = self.agents.index(self.agent_selection) + 1
 
         # Check horizontal locations for win
         column_count = 7
