@@ -1,9 +1,12 @@
-from ._mpe_utils.simple_env import SimpleEnv
+from ._mpe_utils.simple_env import SimpleEnv, make_env
 from .scenarios.simple import Scenario
 
 
-class env(SimpleEnv):
+class raw_env(SimpleEnv):
     def __init__(self, seed=None, max_frames=100):
         scenario = Scenario()
         world = scenario.make_world()
-        super(env, self).__init__(scenario, world, max_frames, seed)
+        super().__init__(scenario, world, max_frames, seed)
+
+
+env = make_env(raw_env)
