@@ -1,4 +1,4 @@
-from .api_test import test_obervation
+from .api_test import test_observation
 import random
 from copy import copy
 
@@ -16,7 +16,7 @@ def bombardment_test(env, cycles=10000):
                 action = env.action_spaces[agent].sample()
             next_observe = env.step(action)
             assert env.observation_spaces[agent].contains(prev_observe), "Agent's observation is outside of its observation space"
-            test_obervation(prev_observe, observation_0)
+            test_observation(prev_observe, observation_0)
             prev_observe = next_observe
             if all(env.dones.values()):
                 prev_observe = env.reset()
